@@ -6,7 +6,7 @@
 
 #include <config-kitinerary.h>
 #include <knowledgedb/airportdb.h>
-#include <knowledgedb/timezonedb.h>
+#include <knowledgedb/timezonedb.cpp>
 
 #include <KItinerary/LocationUtil>
 
