@@ -7,6 +7,7 @@
 #include "extractorvalidator.h"
 #include "validator-logging.h"
 
+#include <KItinerary/BoatTrip>
 #include <KItinerary/BusTrip>
 #include <KItinerary/Event>
 #include <KItinerary/Flight>
@@ -15,6 +16,7 @@
 #include <KItinerary/RentalCar>
 #include <KItinerary/Reservation>
 #include <KItinerary/Taxi>
+#include <KItinerary/Ticket>
 #include <KItinerary/TrainTrip>
 #include <KItinerary/Visit>
 
@@ -33,11 +35,13 @@ public:
     bool filterFlight(const Flight &flight) const;
     bool filterTrainTrip(const TrainTrip &trip) const;
     bool filterBusTrip(const BusTrip &trip) const;
+    bool filterBoatTrip(const BoatTrip &trip) const;
     bool filterEvent(const Event &event) const;
     bool filterFoodReservation(const FoodEstablishmentReservation &res) const;
     bool filterLocalBusiness(const LocalBusiness &business) const;
     bool filterReservation(const Reservation &res) const;
     bool filterProgramMembership(const ProgramMembership &program) const;
+    bool filterTicket(const Ticket &ticket) const;
 
     std::vector<const QMetaObject*> m_acceptedTypes;
     bool m_onlyComplete = true;
@@ -81,25 +85,33 @@ bool ExtractorValidatorPrivate::filterFlight(const Flight &flight) const
 }
 
 template <typename T>
-static bool filterTrainOrBusStation(const T &station)
+static bool filterPlace(const T &place)
 {
-    return !station.name().isEmpty();
+    return !place.name().isEmpty();
 }
 
 bool ExtractorValidatorPrivate::filterTrainTrip(const TrainTrip &trip) const
 {
-    return filterTrainOrBusStation(trip.departureStation())
-           && filterTrainOrBusStation(trip.arrivalStation())
+    return filterPlace(trip.departureStation())
+           && filterPlace(trip.arrivalStation())
            && trip.departureDay().isValid();
 }
 
 bool ExtractorValidatorPrivate::filterBusTrip(const BusTrip &trip) const
 {
-    return filterTrainOrBusStation(trip.departureBusStop())
-           && filterTrainOrBusStation(trip.arrivalBusStop())
+    return filterPlace(trip.departureBusStop())
+           && filterPlace(trip.arrivalBusStop())
            && trip.departureTime().isValid() && trip.arrivalTime().isValid();
 }
 
+bool ExtractorValidatorPrivate::filterBoatTrip(const BoatTrip &trip) const
+{
+    return filterPlace(trip.departureBoatTerminal())
+        && filterPlace(trip.arrivalBoatTerminal())
+        && trip.departureTime().isValid()
+        && trip.arrivalTime().isValid();
+}
+
 bool ExtractorValidatorPrivate::filterEvent(const Event &event) const
 {
     return !event.name().isEmpty() && event.startDate().isValid();
@@ -139,6 +151,11 @@ bool ExtractorValidatorPrivate::filterProgramMembership(const ProgramMembership
     return (!program.membershipNumber().isEmpty() || !program.token().isEmpty()) && !program.programName().isEmpty();
 }
 
+bool ExtractorValidatorPrivate::filterTicket(const Ticket &ticket) const
+{
+    return !ticket.ticketToken().isEmpty() && !ticket.name().isEmpty();
+}
+
 template <typename T, bool (ExtractorValidatorPrivate::*F)(const T&) const>
 static inline bool callFilterWithType(const ExtractorValidatorPrivate *d, const QVariant &v)
 {
@@ -154,12 +171,14 @@ struct {
     FILTER(Flight, filterFlight),
     FILTER(TrainTrip, filterTrainTrip),
     FILTER(BusTrip, filterBusTrip),
+    FILTER(BoatTrip, filterBoatTrip),
     FILTER(KItinerary::Event, filterEvent),
     FILTER(LocalBusiness, filterLocalBusiness),
     FILTER(FoodEstablishmentReservation, filterFoodReservation),
     FILTER(LodgingReservation, filterLodgingReservation),
     FILTER(Reservation, filterReservation),
     FILTER(ProgramMembership, filterProgramMembership),
+    FILTER(Ticket, filterTicket),
 };
 #undef FILTER
 
