@@ -10,6 +10,7 @@
 #include <KItinerary/JsonLdDocument>
 #include <KItinerary/Uic9183Parser>
 #include <KItinerary/Rct2Ticket>
+#include <era/fcbticket.h>
 #include <uic9183/uic9183head.h>
 #include <uic9183/vendor0080block.h>
 
@@ -50,23 +51,14 @@ void Uic9183DocumentProcessor::expandNode(ExtractorDocumentNode &node, [[maybe_u
     // while that is usually correct it cannot contain a time zone, unlike the (often) enclosing PDF document´
     if (!node.contextDateTime().isValid()) {
         const auto p = node.content<Uic9183Parser>();
-        const auto u_head = p.findBlock<Uic9183Head>();
-        node.setContextDateTime(u_head.issuingDateTime());
+        if (const auto u_flex = p.findBlock<Fcb::UicRailTicketData>(); u_flex.isValid()) {
+            node.setContextDateTime(u_flex.issuingDetail.issueingDateTime());
+        } else if (const auto u_head = p.findBlock<Uic9183Head>(); u_head.isValid()) {
+            node.setContextDateTime(u_head.issuingDateTime());
+        }
     }
 }
 
-static QJsonValue makeStation(const QString &name)
-{
-    if (name.isEmpty()) {
-        return {};
-    }
-
-    QJsonObject station;
-    station.insert(QStringLiteral("@type"), QLatin1String("TrainStation"));
-    station.insert(QStringLiteral("name"), name);
-    return station;
-}
-
 static bool isValidTrip(const QJsonObject &trip)
 {
     if (trip.size() <= 1) {
@@ -74,8 +66,8 @@ static bool isValidTrip(const QJsonObject &trip)
     }
 
     return trip.contains(QLatin1String("departureDay"))
-        && trip.value(QLatin1String("departureStation")).isObject()
-        && trip.value(QLatin1String("arrivalStation")).isObject();
+        && !trip.value(QLatin1String("departureStation")).toObject().value(QLatin1String("name")).toString().isEmpty()
+        && !trip.value(QLatin1String("arrivalStation")).toObject().value(QLatin1String("name")).toString().isEmpty();
 }
 
 void Uic9183DocumentProcessor::preExtract(ExtractorDocumentNode &node, [[maybe_unused]] const ExtractorEngine *engine) const
@@ -86,9 +78,6 @@ void Uic9183DocumentProcessor::preExtract(ExtractorDocumentNode &node, [[maybe_u
     trip.insert(QLatin1String("@type"), QLatin1String("TrainTrip"));
     QJsonObject returnTrip;
     returnTrip.insert(QLatin1String("@type"), QLatin1String("TrainTrip"));
-    QJsonObject provider;
-    provider.insert(QLatin1String("@type"), QLatin1String("Organization"));
-    provider.insert(QLatin1String("identifier"), QJsonValue(QLatin1String("uic:") + p.carrierId()));
 
     QJsonObject seat;
     seat.insert(QLatin1String("@type"), QLatin1String("Seat"));
@@ -110,8 +99,8 @@ void Uic9183DocumentProcessor::preExtract(ExtractorDocumentNode &node, [[maybe_u
             case Rct2Ticket::Transport:
             case Rct2Ticket::Upgrade:
             {
-                trip.insert(QStringLiteral("departureStation"), makeStation(rct2.outboundDepartureStation()));
-                trip.insert(QStringLiteral("arrivalStation"), makeStation(rct2.outboundArrivalStation()));
+                trip.insert(QStringLiteral("departureStation"), JsonLdDocument::toJson(p.outboundDepartureStation()));
+                trip.insert(QStringLiteral("arrivalStation"), JsonLdDocument::toJson(p.outboundArrivalStation()));
 
                 if (rct2.outboundDepartureTime().isValid()) {
                     trip.insert(QStringLiteral("departureDay"),  JsonLdDocument::toJsonValue(rct2.outboundDepartureTime().date()));
@@ -124,9 +113,9 @@ void Uic9183DocumentProcessor::preExtract(ExtractorDocumentNode &node, [[maybe_u
                     trip.insert(QStringLiteral("arrivalTime"),  JsonLdDocument::toJsonValue(rct2.outboundArrivalTime()));
                 }
 
-                if (rct2.type() == Rct2Ticket::Transport && !rct2.returnDepartureStation().isEmpty()) {
-                    returnTrip.insert(QStringLiteral("departureStation"), makeStation(rct2.returnDepartureStation()));
-                    returnTrip.insert(QStringLiteral("arrivalStation"), makeStation(rct2.returnArrivalStation()));
+                if (rct2.type() == Rct2Ticket::Transport && !p.returnDepartureStation().name().isEmpty()) {
+                    returnTrip.insert(QStringLiteral("departureStation"), JsonLdDocument::toJson(p.returnDepartureStation()));
+                    returnTrip.insert(QStringLiteral("arrivalStation"), JsonLdDocument::toJson(p.returnArrivalStation()));
 
                     if (rct2.returnDepartureTime().isValid()) {
                         returnTrip.insert(QStringLiteral("departureDay"),  JsonLdDocument::toJsonValue(rct2.returnDepartureTime().date()));
@@ -166,7 +155,7 @@ void Uic9183DocumentProcessor::preExtract(ExtractorDocumentNode &node, [[maybe_u
 
     // we have enough for a full TrainReservation result
     if (isValidTrip(trip)) {
-        trip.insert(QLatin1String("provider"), provider);
+        trip.insert(QLatin1String("provider"), JsonLdDocument::toJson(p.issuer()));
 
         QJsonArray results;
         QJsonObject res;
@@ -188,7 +177,7 @@ void Uic9183DocumentProcessor::preExtract(ExtractorDocumentNode &node, [[maybe_u
 
     // only Ticket
     ticket.insert(QStringLiteral("name"), p.name());
-    ticket.insert(QStringLiteral("issuedBy"), provider);
+    ticket.insert(QStringLiteral("issuedBy"), JsonLdDocument::toJson(p.issuer()));
     ticket.insert(QStringLiteral("ticketNumber"), p.pnr());
     ticket.insert(QStringLiteral("underName"), JsonLdDocument::toJson(p.person()));
     if (p.validFrom().isValid()) {
